package com.sun.xml.internal.ws.api.policy.subject;

import com.sun.xml.internal.ws.resources.BindingApiMessages;
import javax.xml.namespace.QName;
import com.sun.istack.internal.logging.Logger;

public class BindingSubject
{
    private static final Logger LOGGER;
    private final QName name;
    private final BindingSubject.WsdlMessageType messageType;
    private final BindingSubject.WsdlNameScope nameScope;
    private final BindingSubject parent;
    
    BindingSubject(QName name, BindingSubject.WsdlNameScope scope, BindingSubject parent) {
        this(name, BindingSubject.WsdlMessageType.NO_MESSAGE, scope, parent);
    }
    
    BindingSubject(QName name, BindingSubject.WsdlMessageType messageType, BindingSubject.WsdlNameScope scope, BindingSubject parent) {
        this.name = name;
        this.messageType = messageType;
        this.nameScope = scope;
        this.parent = parent;
    }
    
    public static BindingSubject createBindingSubject(QName bindingName) {
        return new BindingSubject(bindingName, BindingSubject.WsdlNameScope.ENDPOINT, null);
    }
    
    public static BindingSubject createOperationSubject(QName bindingName, QName operationName) {
        BindingSubject bindingSubject = createBindingSubject(bindingName);
        return new BindingSubject(operationName, BindingSubject.WsdlNameScope.OPERATION, bindingSubject);
    }
    
    public static BindingSubject createInputMessageSubject(QName bindingName, QName operationName, QName messageName) {
        BindingSubject operationSubject = createOperationSubject(bindingName, operationName);
        return new BindingSubject(messageName, BindingSubject.WsdlMessageType.INPUT, BindingSubject.WsdlNameScope.MESSAGE, operationSubject);
    }
    
    public static BindingSubject createOutputMessageSubject(QName bindingName, QName operationName, QName messageName) {
        BindingSubject operationSubject = createOperationSubject(bindingName, operationName);
        return new BindingSubject(messageName, BindingSubject.WsdlMessageType.OUTPUT, BindingSubject.WsdlNameScope.MESSAGE, operationSubject);
    }
    
    public static BindingSubject createFaultMessageSubject(QName bindingName, QName operationName, QName messageName) {
        if (messageName == null) {
            throw (IllegalArgumentException)BindingSubject.LOGGER.logSevereException(new IllegalArgumentException(BindingApiMessages.BINDING_API_NO_FAULT_MESSAGE_NAME()));
        }
        BindingSubject operationSubject = createOperationSubject(bindingName, operationName);
        return new BindingSubject(messageName, BindingSubject.WsdlMessageType.FAULT, BindingSubject.WsdlNameScope.MESSAGE, operationSubject);
    }
    
    public QName getName() {
        return this.name;
    }
    
    public BindingSubject getParent() {
        return this.parent;
    }
    
    public boolean isBindingSubject() {
        return this.nameScope == BindingSubject.WsdlNameScope.ENDPOINT && this.parent == null;
    }
    
    public boolean isOperationSubject() {
        return this.nameScope == BindingSubject.WsdlNameScope.OPERATION && this.parent != null && this.parent.isBindingSubject();
    }
    
    public boolean isMessageSubject() {
        return this.nameScope == BindingSubject.WsdlNameScope.MESSAGE && this.parent != null && this.parent.isOperationSubject();
    }
    
    public boolean isInputMessageSubject() {
        return this.isMessageSubject() && this.messageType == BindingSubject.WsdlMessageType.INPUT;
    }
    
    public boolean isOutputMessageSubject() {
        return this.isMessageSubject() && this.messageType == BindingSubject.WsdlMessageType.OUTPUT;
    }
    
    public boolean isFaultMessageSubject() {
        return this.isMessageSubject() && this.messageType == BindingSubject.WsdlMessageType.FAULT;
    }
    
    @Override
    public boolean equals(Object that) {
        if (this == that) {
            return true;
        }
        if (that == null || !(that instanceof BindingSubject)) {
            return false;
        }
        BindingSubject thatSubject = (BindingSubject)that;
        boolean isEqual = true;
        isEqual = (isEqual && ((this.name != null) ? this.name.equals(thatSubject.name) : (thatSubject.name == null)));
        isEqual = (isEqual && this.messageType.equals(thatSubject.messageType));
        isEqual = (isEqual && this.nameScope.equals(thatSubject.nameScope));
        isEqual = (isEqual && ((this.parent != null) ? this.parent.equals(thatSubject.parent) : (thatSubject.parent == null)));
        return isEqual;
    }
    
    @Override
    public int hashCode() {
        int result = 23;
        result = 29 * result + ((this.name == null) ? 0 : this.name.hashCode());
        result = 29 * result + this.messageType.hashCode();
        result = 29 * result + this.nameScope.hashCode();
        result = 29 * result + ((this.parent == null) ? 0 : this.parent.hashCode());
        return result;
    }
    
    @Override
    public String toString() {
        StringBuilder result = new StringBuilder("BindingSubject[");
        result.append(this.name).append(", ").append(this.messageType);
        result.append(", ").append(this.nameScope).append(", ").append(this.parent);
        return result.append("]").toString();
    }
    
    static {
        LOGGER = Logger.getLogger(BindingSubject.class);
    }
    
    private enum WsdlMessageType
    {
        NO_MESSAGE, 
        INPUT, 
        OUTPUT, 
        FAULT;
    }
    
    private enum WsdlNameScope
    {
        SERVICE, 
        ENDPOINT, 
        OPERATION, 
        MESSAGE;
    }
}
